@@ -1,7 +1,7 @@
 /*
  * augeas.c: the core data structure for storing key/value pairs
  *
- * Copyright (C) 2007-2015 David Lutterkort
+ * Copyright (C) 2007-2016 David Lutterkort
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -697,7 +697,7 @@ static void tree_rm_dirty_files(struct augeas *aug, struct tree *tree) {
     if (!tree->dirty)
         return;
 
-    if ((p = tree_child(tree, "path")) != NULL) {
+    if (tree->file && ((p = tree_child(tree, "path")) != NULL)) {
         tree_unlink(aug, tree_fpath(aug, p->value));
         tree_unlink(aug, tree);
     } else {
@@ -910,7 +910,7 @@ int aug_defvar(augeas *aug, const char *name, const char *expr) {
 
 int aug_defnode(augeas *aug, const char *name, const char *expr,
                 const char *value, int *created) {
-    struct pathx *p;
+    struct pathx *p = NULL;
     int result = -1;
     int r, cr;
     struct tree *tree;
@@ -951,12 +951,10 @@ int aug_defnode(augeas *aug, const char *name, const char *expr,
     }
 
  done:
+ error:
     free_pathx(p);
     api_exit(aug);
     return result;
- error:
-    api_exit(aug);
-    return -1;
 }
 
 struct tree *tree_set(struct pathx *p, const char *value) {
@@ -974,8 +972,8 @@ struct tree *tree_set(struct pathx *p, const char *value) {
 }
 
 int aug_set(struct augeas *aug, const char *path, const char *value) {
-    struct pathx *p;
-    int result;
+    struct pathx *p = NULL;
+    int result = -1;
 
     api_entry(aug);
 
@@ -988,13 +986,10 @@ int aug_set(struct augeas *aug, const char *path, const char *value) {
     ERR_BAIL(aug);
 
     result = tree_set(p, value) == NULL ? -1 : 0;
+ error:
     free_pathx(p);
-
     api_exit(aug);
     return result;
- error:
-    api_exit(aug);
-    return -1;
 }
 
 int aug_setm(struct augeas *aug, const char *base,
@@ -1044,6 +1039,8 @@ int aug_setm(struct augeas *aug, const char *base,
     }
 
  done:
+    free_pathx(bx);
+    free_pathx(sx);
     api_exit(aug);
     return result;
  error:
@@ -1193,7 +1190,7 @@ int tree_rm(struct pathx *p) {
 
 int aug_rm(struct augeas *aug, const char *path) {
     struct pathx *p = NULL;
-    int result;
+    int result = -1;
 
     api_entry(aug);
 
@@ -1201,14 +1198,11 @@ int aug_rm(struct augeas *aug, const char *path) {
     ERR_BAIL(aug);
 
     result = tree_rm(p);
-    free_pathx(p);
-    ERR_BAIL(aug);
 
+ error:
+    free_pathx(p);
     api_exit(aug);
     return result;
- error:
-    api_exit(aug);
-    return -1;
 }
 
 int aug_span(struct augeas *aug, const char *path, char **filename,
@@ -1465,6 +1459,29 @@ int aug_match(const struct augeas *aug, const char *pathin, char ***matches) {
     return -1;
 }
 
+/* XFM1 and XFM2 can both be used to save the same file. That is an error
+   only if the two lenses in the two transforms are actually different. */
+static int check_save_dup(struct augeas *aug, const char *path,
+                          struct tree *xfm1, struct tree *xfm2) {
+    int result = 0;
+    struct lens *l1 = xfm_lens(aug, xfm1, NULL);
+    struct lens *l2 = xfm_lens(aug, xfm2, NULL);
+    if (l1 != l2) {
+        const char *filename = path + strlen(AUGEAS_FILES_TREE) + 1;
+        transform_file_error(aug, "mxfm_save", filename,
+                             "Lenses %s and %s could be used to save this file",
+                             xfm_lens_name(xfm1),
+                             xfm_lens_name(xfm2));
+        ERR_REPORT(aug, AUG_EMXFM,
+                   "Path %s transformable by lens %s and %s",
+                   path,
+                   xfm_lens_name(xfm1),
+                   xfm_lens_name(xfm2));
+        result = -1;
+    }
+    return result;
+}
+
 static int tree_save(struct augeas *aug, struct tree *tree,
                      const char *path) {
     int result = 0;
@@ -1489,18 +1506,7 @@ static int tree_save(struct augeas *aug, struct tree *tree,
                     if (transform == NULL || transform == xfm) {
                         transform = xfm;
                     } else {
-                        const char *filename =
-                            tpath + strlen(AUGEAS_FILES_TREE) + 1;
-                        transform_file_error(aug, "mxfm_save", filename,
-                           "Lenses %s and %s could be used to save this file",
-                                             xfm_lens_name(transform),
-                                             xfm_lens_name(xfm));
-                        ERR_REPORT(aug, AUG_EMXFM,
-                                   "Path %s transformable by lens %s and %s",
-                                   tpath,
-                                   xfm_lens_name(transform),
-                                   xfm_lens_name(xfm));
-                        result = -1;
+                        result = check_save_dup(aug, tpath, transform, xfm);
                     }
                 }
             }
@@ -1563,6 +1569,7 @@ static int unlink_removed_files(struct augeas *aug,
             if (pathx_parse(tm, err_of_aug(aug), file_nodes, true,
                             aug->symtab, NULL, &px) != PATHX_NOERROR) {
                 result = -1;
+                free_pathx(px);
                 continue;
             }
             for (struct tree *t = pathx_first(px);
@@ -1701,8 +1708,10 @@ int dump_tree(FILE *out, struct tree *tree) {
     struct pathx *p;
     int result;
 
-    if (pathx_parse(tree, NULL, "/*", true, NULL, NULL, &p) != PATHX_NOERROR)
+    if (pathx_parse(tree, NULL, "/*", true, NULL, NULL, &p) != PATHX_NOERROR) {
+        free_pathx(p);
         return -1;
+    }
 
     result = print_tree(out, p, 1);
     free_pathx(p);
@@ -1861,8 +1870,8 @@ int aug_text_store(augeas *aug, const char *lens, const char *node,
 
     /* Validate PATH is syntactically correct */
     p = pathx_aug_parse(aug, aug->origin, tree_root_ctx(aug), path, true);
-    ERR_BAIL(aug);
     free_pathx(p);
+    ERR_BAIL(aug);
 
     r = aug_get(aug, node, &src);
     ERR_BAIL(aug);
@@ -1917,14 +1926,16 @@ int aug_text_retrieve(struct augeas *aug, const char *lens,
 
 int aug_to_xml(const struct augeas *aug, const char *pathin,
                xmlNode **xmldoc, unsigned int flags) {
-    struct pathx *p;
-    int result;
+    struct pathx *p = NULL;
+    int result = -1;
 
     api_entry(aug);
 
     ARG_CHECK(flags != 0, aug, "aug_to_xml: FLAGS must be 0");
     ARG_CHECK(xmldoc == NULL, aug, "aug_to_xml: XMLDOC must be non-NULL");
 
+    *xmldoc = NULL;
+
     if (pathin == NULL || strlen(pathin) == 0 || strcmp(pathin, "/") == 0) {
         pathin = "/*";
     }
@@ -1933,15 +1944,11 @@ int aug_to_xml(const struct augeas *aug, const char *pathin,
     ERR_BAIL(aug);
     result = tree_to_xml(p, xmldoc, pathin);
     ERR_THROW(result < 0, aug, AUG_ENOMEM, NULL);
+error:
     free_pathx(p);
     api_exit(aug);
 
     return result;
- error:
-    if (xmldoc !=NULL)
-        *xmldoc = NULL;
-    api_exit(aug);
-    return -1;
 }
 
 int aug_transform(struct augeas *aug, const char *lens,
@@ -2021,7 +2028,7 @@ int aug_escape_name(augeas *aug, const char *in, char **out) {
 }
 
 int aug_load_file(struct augeas *aug, const char *file) {
-    int result = -1;
+    int result = -1, r;
     struct tree *meta = tree_child_cr(aug->origin, s_augeas);
     struct tree *load = tree_child_cr(meta, s_load);
     char *tree_path = NULL;
@@ -2044,7 +2051,9 @@ int aug_load_file(struct augeas *aug, const char *file) {
 
     /* Mark the nodes we just loaded as clean so they won't get saved
        without additional modifications */
-    xasprintf(&tree_path, "/files/%s", file);
+    r = xasprintf(&tree_path, "/files/%s", file);
+    ERR_NOMEM(r < 0, aug);
+
     struct tree *t = tree_fpath(aug, tree_path);
     if (t != NULL) {
         tree_clean(t);
@@ -2059,7 +2068,7 @@ error:
 
 int aug_print(const struct augeas *aug, FILE *out, const char *pathin) {
     struct pathx *p;
-    int result;
+    int result = -1;
 
     api_entry(aug);
 
@@ -2071,13 +2080,10 @@ int aug_print(const struct augeas *aug, FILE *out, const char *pathin) {
     ERR_BAIL(aug);
 
     result = print_tree(out, p, 0);
+ error:
     free_pathx(p);
-
     api_exit(aug);
     return result;
- error:
-    api_exit(aug);
-    return -1;
 }
 
 void aug_close(struct augeas *aug) {
