@@ -1,7 +1,7 @@
 /*
  * augrun.c: command interpreter for augeas
  *
- * Copyright (C) 2011-2015 David Lutterkort
+ * Copyright (C) 2011-2016 David Lutterkort
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -1232,7 +1232,6 @@ static const char *err_get(struct augeas *aug,
 static void cmd_errors(struct command *cmd) {
     char **matches = NULL;
     int cnt = 0;
-    char *filename = NULL;
     struct augeas *aug = cmd->aug;
 
     cnt = aug_match(aug, "/augeas//error", &matches);
@@ -1258,7 +1257,7 @@ static void cmd_errors(struct command *cmd) {
         aug_get(aug, match, &kind);
         ERR_BAIL(aug);
 
-        filename = err_filename(match);
+        char *filename = err_filename(match);
         ERR_NOMEM(filename == NULL, aug);
 
         if (i>0)
@@ -1272,6 +1271,7 @@ static void cmd_errors(struct command *cmd) {
         } else {
             fprintf(cmd->out, "Error in %s (%s)\n", filename, kind);
         }
+        FREE(filename);
 
         if (msg != NULL)
             fprintf(cmd->out, "  %s\n", msg);
@@ -1288,7 +1288,6 @@ static void cmd_errors(struct command *cmd) {
     for (int i=0; i < cnt; i++)
         free(matches[i]);
     free(matches);
-    free(filename);
 }
 
 static const struct command_opt_def cmd_errors_opts[] = {
@@ -1450,14 +1449,14 @@ int aug_srun(augeas *aug, FILE *out, const char *text) {
 
     api_entry(aug);
 
-    if (text == NULL)
-        goto done;
-
     MEMZERO(&cmd, 1);
     cmd.aug = aug;
     cmd.error = aug->error;
     cmd.out = out;
 
+    if (text == NULL)
+        goto done;
+
     while (*text != '\0' && result >= 0) {
         eol = strchrnul(text, '\n');
         while (isspace(*text) && text < eol) text++;
@@ -1489,6 +1488,7 @@ int aug_srun(augeas *aug, FILE *out, const char *text) {
         text = (*eol == '\0') ? eol : eol + 1;
     }
  done:
+    free_command_opts(&cmd);
     FREE(line);
 
     api_exit(aug);
